@@ -1,13 +1,3 @@
-#include <sys/stat.h>
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <errno.h>
-#include <string.h>
-#include <fcntl.h>
-#include <assert.h>
-
 #include "buffer.h"
 #include "server.h"
 #include "log.h"
@@ -25,6 +15,16 @@
 
 #include "inet_ntop_cache.h"
 
+#include <sys/stat.h>
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <unistd.h>
+#include <errno.h>
+#include <string.h>
+#include <fcntl.h>
+#include <assert.h>
+
 #ifdef USE_OPENSSL
 # include <openssl/ssl.h>
 # include <openssl/err.h>
@@ -192,7 +192,7 @@ static void dump_packet(const unsigned char *data, size_t len) {
 
 static int connection_handle_read_ssl(server *srv, connection *con) {
 #ifdef USE_OPENSSL
-	int r, ssl_err, len;
+	int r, ssl_err, len, count = 0;
 	buffer *b = NULL;
 
 	if (!con->conf.is_ssl) return -1;
@@ -221,13 +221,15 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 		       	/* we move the buffer to the chunk-queue, no need to free it */
 
 			chunkqueue_append_buffer_weak(con->read_queue, b);
+			count += len;
 			con->bytes_read += len;
 			b = NULL;
 		}
-	} while (len > 0);
+	} while (len > 0 && count < MAX_READ_LIMIT);
 
 
 	if (len < 0) {
+		int oerrno = errno;
 		switch ((r = SSL_get_error(con->ssl, len))) {
 		case SSL_ERROR_WANT_READ:
 		case SSL_ERROR_WANT_WRITE:
@@ -257,11 +259,11 @@ static int connection_handle_read_ssl(server *srv, connection *con) {
 						r, ERR_error_string(ssl_err, NULL));
 			}
 
-			switch(errno) {
+			switch(oerrno) {
 			default:
 				log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL:",
-						len, r, errno,
-						strerror(errno));
+						len, r, oerrno,
+						strerror(oerrno));
 				break;
 			}
 
@@ -334,6 +336,7 @@ static int connection_handle_read(server *srv, connection *con) {
 		b = chunkqueue_get_append_buffer(con->read_queue);
 		buffer_prepare_copy(b, 4 * 1024);
 	} else {
+		if (toread > MAX_READ_LIMIT) toread = MAX_READ_LIMIT;
 		b = chunkqueue_get_append_buffer(con->read_queue);
 		buffer_prepare_copy(b, toread + 1);
 	}
@@ -664,6 +667,9 @@ connection *connection_init(server *srv) {
 	CLEAN(server_name);
 	CLEAN(error_handler);
 	CLEAN(dst_addr_buf);
+#if defined USE_OPENSSL && ! defined OPENSSL_NO_TLSEXT
+	CLEAN(tlsext_server_name);
+#endif
 
 #undef CLEAN
 	con->write_queue = chunkqueue_init();
@@ -728,6 +734,9 @@ void connections_free(server *srv) {
 		CLEAN(server_name);
 		CLEAN(error_handler);
 		CLEAN(dst_addr_buf);
+#if defined USE_OPENSSL && ! defined OPENSSL_NO_TLSEXT
+		CLEAN(tlsext_server_name);
+#endif
 #undef CLEAN
 		free(con->plugin_ctx);
 		free(con->cond_cache);
@@ -779,13 +788,13 @@ int connection_reset(server *srv, connection *con) {
 	CLEAN(request.pathinfo);
 	CLEAN(request.request);
 
-	CLEAN(request.orig_uri);
+	/* CLEAN(request.orig_uri); */
 
 	CLEAN(uri.scheme);
-	CLEAN(uri.authority);
-	CLEAN(uri.path);
+	/* CLEAN(uri.authority); */
+	/* CLEAN(uri.path); */
 	CLEAN(uri.path_raw);
-	CLEAN(uri.query);
+	/* CLEAN(uri.query); */
 
 	CLEAN(physical.doc_root);
 	CLEAN(physical.path);
@@ -858,7 +867,7 @@ int connection_reset(server *srv, connection *con) {
  *
  * we get called by the state-engine and by the fdevent-handler
  */
-int connection_handle_read_state(server *srv, connection *con)  {
+static int connection_handle_read_state(server *srv, connection *con)  {
 	connection_state_t ostate = con->state;
 	chunk *c, *last_chunk;
 	off_t last_offset;
@@ -1156,7 +1165,7 @@ int connection_handle_read_state(server *srv, connection *con)  {
 	return 0;
 }
 
-handler_t connection_handle_fdevent(void *s, void *context, int revents) {
+static handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 	server     *srv = (server *)s;
 	connection *con = context;
 
@@ -1180,7 +1189,7 @@ handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 		/* FIXME: revents = 0x19 still means that we should read from the queue */
 		if (revents & FDEVENT_HUP) {
 			if (con->state == CON_STATE_CLOSE) {
-				con->close_timeout_ts = 0;
+				con->close_timeout_ts = srv->cur_ts - (HTTP_LINGER_TIMEOUT+1);
 			} else {
 				/* sigio reports the wrong event here
 				 *
@@ -1247,9 +1256,11 @@ handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 			/* */
 			read(con->fd, buf, sizeof(buf));
 		} else {
-			/* nothing to read */
-
-			con->close_timeout_ts = 0;
+			/* nothing to read - yet.  But that doesn't
+			 * mean something won't show up in our buffers
+			 * sometime soon, so we can't quite until
+			 * poll() gives us the HUP notification.
+			 */
 		}
 	}
 
@@ -1338,6 +1349,9 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 				return NULL;
 			}
 
+#ifndef OPENSSL_NO_TLSEXT
+			SSL_set_app_data(con->ssl, con);
+#endif
 			SSL_set_accept_state(con->ssl);
 			con->conf.is_ssl=1;
 
@@ -1398,6 +1412,11 @@ int connection_state_machine(server *srv, connection *con) {
 						"state for fd", con->fd, connection_get_state(con->state));
 			}
 
+			buffer_reset(con->uri.authority);
+			buffer_reset(con->uri.path);
+			buffer_reset(con->uri.query);
+			buffer_reset(con->request.orig_uri);
+
 			if (http_request_parse(srv, con)) {
 				/* we have to read some data from the POST request */
 
@@ -1561,7 +1580,12 @@ int connection_state_machine(server *srv, connection *con) {
 					}
 				}
 #endif
-				connection_close(srv, con);
+				if ((0 == shutdown(con->fd, SHUT_WR))) {
+					con->close_timeout_ts = srv->cur_ts;
+					connection_set_state(srv, con, CON_STATE_CLOSE);
+				} else {
+					connection_close(srv, con);
+				}
 
 				srv->con_closed++;
 			}
@@ -1586,28 +1610,31 @@ int connection_state_machine(server *srv, connection *con) {
 						"state for fd", con->fd, connection_get_state(con->state));
 			}
 
-			if (con->keep_alive) {
-				if (ioctl(con->fd, FIONREAD, &b)) {
-					log_error_write(srv, __FILE__, __LINE__, "ss",
-							"ioctl() failed", strerror(errno));
-				}
-				if (b > 0) {
-					char buf[1024];
-					log_error_write(srv, __FILE__, __LINE__, "sdd",
-							"CLOSE-read()", con->fd, b);
+			/* we have to do the linger_on_close stuff regardless
+			 * of con->keep_alive; even non-keepalive sockets may
+			 * still have unread data, and closing before reading
+			 * it will make the client not see all our output.
+			 */
+			if (ioctl(con->fd, FIONREAD, &b)) {
+				log_error_write(srv, __FILE__, __LINE__, "ss",
+					"ioctl() failed", strerror(errno));
+			}
+			if (b > 0) {
+				char buf[1024];
+				log_error_write(srv, __FILE__, __LINE__, "sdd",
+						"CLOSE-read()", con->fd, b);
 
-					/* */
-					read(con->fd, buf, sizeof(buf));
-				} else {
-					/* nothing to read */
-
-					con->close_timeout_ts = 0;
-				}
+				/* */
+				read(con->fd, buf, sizeof(buf));
 			} else {
-				con->close_timeout_ts = 0;
+				/* nothing to read - yet.  But that doesn't
+				 * mean something won't show up in our buffers
+				 * sometime soon, so we can't quite until
+				 * poll() gives us the HUP notification.
+				 */
 			}
 
-			if (srv->cur_ts - con->close_timeout_ts > 1) {
+			if (srv->cur_ts - con->close_timeout_ts > HTTP_LINGER_TIMEOUT) {
 				connection_close(srv, con);
 
 				if (srv->srvconf.log_state_handling) {
@@ -1722,7 +1749,7 @@ int connection_state_machine(server *srv, connection *con) {
 				case HANDLER_FINISHED:
 					break;
 				default:
-					log_error_write(srv, __FILE__, __LINE__, "");
+					log_error_write(srv, __FILE__, __LINE__, "sd", "unhandling return value", r);
 					break;
 				}
 				break;
@@ -1731,8 +1758,7 @@ int connection_state_machine(server *srv, connection *con) {
 			connection_reset(srv, con);
 
 			/* close the connection */
-			if ((con->keep_alive == 1) &&
-			    (0 == shutdown(con->fd, SHUT_WR))) {
+			if ((0 == shutdown(con->fd, SHUT_WR))) {
 				con->close_timeout_ts = srv->cur_ts;
 				connection_set_state(srv, con, CON_STATE_CLOSE);
 
