@@ -1,4 +1,10 @@
-#define _GNU_SOURCE
+#include "server.h"
+#include "connections.h"
+#include "response.h"
+#include "connections.h"
+#include "log.h"
+
+#include "plugin.h"
 #include <sys/types.h>
 
 #include <fcntl.h>
@@ -9,13 +15,6 @@
 #include <errno.h>
 #include <time.h>
 
-#include "server.h"
-#include "connections.h"
-#include "response.h"
-#include "connections.h"
-#include "log.h"
-
-#include "plugin.h"
 #ifdef HAVE_FORK
 /* no need for waitpid if we don't have fork */
 #include <sys/wait.h>
@@ -91,7 +90,7 @@ FREE_FUNC(mod_rrd_free) {
 	return HANDLER_GO_ON;
 }
 
-int mod_rrd_create_pipe(server *srv, plugin_data *p) {
+static int mod_rrd_create_pipe(server *srv, plugin_data *p) {
 #ifdef HAVE_FORK
 	pid_t pid;
 
@@ -139,9 +138,7 @@ int mod_rrd_create_pipe(server *srv, plugin_data *p) {
 
 		args[i++] = p->conf.path_rrdtool_bin->ptr;
 		args[i++] = dash;
-		args[i++] = NULL;
-
-		openDevNull(STDERR_FILENO);
+		args[i  ] = NULL;
 
 		/* we don't need the client socket */
 		for (i = 3; i < 256; i++) {
@@ -230,6 +227,7 @@ static ssize_t safe_read(int fd, void *buf, size_t count) {
 
 static int mod_rrdtool_create_rrd(server *srv, plugin_data *p, plugin_config *s) {
 	struct stat st;
+	int r;
 
 	/* check if DB already exists */
 	if (0 == stat(s->path_rrd->ptr, &st)) {
@@ -239,54 +237,57 @@ static int mod_rrdtool_create_rrd(server *srv, plugin_data *p, plugin_config *s)
 					"not a regular file:", s->path_rrd);
 			return HANDLER_ERROR;
 		}
-	} else {
-		int r ;
-		/* create a new one */
+	}
 
-		buffer_copy_string_len(p->cmd, CONST_STR_LEN("create "));
-		buffer_append_string_buffer(p->cmd, s->path_rrd);
-		buffer_append_string_len(p->cmd, CONST_STR_LEN(
-			" --step 60 "
-			"DS:InOctets:ABSOLUTE:600:U:U "
-			"DS:OutOctets:ABSOLUTE:600:U:U "
-			"DS:Requests:ABSOLUTE:600:U:U "
-			"RRA:AVERAGE:0.5:1:600 "
-			"RRA:AVERAGE:0.5:6:700 "
-			"RRA:AVERAGE:0.5:24:775 "
-			"RRA:AVERAGE:0.5:288:797 "
-			"RRA:MAX:0.5:1:600 "
-			"RRA:MAX:0.5:6:700 "
-			"RRA:MAX:0.5:24:775 "
-			"RRA:MAX:0.5:288:797 "
-			"RRA:MIN:0.5:1:600 "
-			"RRA:MIN:0.5:6:700 "
-			"RRA:MIN:0.5:24:775 "
-			"RRA:MIN:0.5:288:797\n"));
+	/* still create DB if it's empty file */
+	if (st.st_size > 0) {
+		return HANDLER_GO_ON;
+	}
 
-		if (-1 == (r = safe_write(p->write_fd, p->cmd->ptr, p->cmd->used - 1))) {
-			log_error_write(srv, __FILE__, __LINE__, "ss",
-				"rrdtool-write: failed", strerror(errno));
+	/* create a new one */
+	buffer_copy_string_len(p->cmd, CONST_STR_LEN("create "));
+	buffer_append_string_buffer(p->cmd, s->path_rrd);
+	buffer_append_string_len(p->cmd, CONST_STR_LEN(
+		" --step 60 "
+		"DS:InOctets:ABSOLUTE:600:U:U "
+		"DS:OutOctets:ABSOLUTE:600:U:U "
+		"DS:Requests:ABSOLUTE:600:U:U "
+		"RRA:AVERAGE:0.5:1:600 "
+		"RRA:AVERAGE:0.5:6:700 "
+		"RRA:AVERAGE:0.5:24:775 "
+		"RRA:AVERAGE:0.5:288:797 "
+		"RRA:MAX:0.5:1:600 "
+		"RRA:MAX:0.5:6:700 "
+		"RRA:MAX:0.5:24:775 "
+		"RRA:MAX:0.5:288:797 "
+		"RRA:MIN:0.5:1:600 "
+		"RRA:MIN:0.5:6:700 "
+		"RRA:MIN:0.5:24:775 "
+		"RRA:MIN:0.5:288:797\n"));
 
-			return HANDLER_ERROR;
-		}
+	if (-1 == (safe_write(p->write_fd, p->cmd->ptr, p->cmd->used - 1))) {
+		log_error_write(srv, __FILE__, __LINE__, "ss",
+			"rrdtool-write: failed", strerror(errno));
 
-		buffer_prepare_copy(p->resp, 4096);
-		if (-1 == (r = safe_read(p->read_fd, p->resp->ptr, p->resp->size))) {
-			log_error_write(srv, __FILE__, __LINE__, "ss",
-				"rrdtool-read: failed", strerror(errno));
+		return HANDLER_ERROR;
+	}
 
-			return HANDLER_ERROR;
-		}
+	buffer_prepare_copy(p->resp, 4096);
+	if (-1 == (r = safe_read(p->read_fd, p->resp->ptr, p->resp->size))) {
+		log_error_write(srv, __FILE__, __LINE__, "ss",
+			"rrdtool-read: failed", strerror(errno));
 
-		p->resp->used = r;
+		return HANDLER_ERROR;
+	}
 
-		if (p->resp->ptr[0] != 'O' ||
-		    p->resp->ptr[1] != 'K') {
-			log_error_write(srv, __FILE__, __LINE__, "sbb",
-				"rrdtool-response:", p->cmd, p->resp);
+	p->resp->used = r;
 
-			return HANDLER_ERROR;
-		}
+	if (p->resp->ptr[0] != 'O' ||
+		p->resp->ptr[1] != 'K') {
+		log_error_write(srv, __FILE__, __LINE__, "sbb",
+			"rrdtool-response:", p->cmd, p->resp);
+
+		return HANDLER_ERROR;
 	}
 
 	return HANDLER_GO_ON;
@@ -477,6 +478,7 @@ REQUESTDONE_FUNC(mod_rrd_account) {
 	return HANDLER_GO_ON;
 }
 
+int mod_rrdtool_plugin_init(plugin *p);
 int mod_rrdtool_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name        = buffer_init_string("rrd");
