@@ -1,3 +1,14 @@
+#include "response.h"
+#include "keyvalue.h"
+#include "log.h"
+#include "stat_cache.h"
+#include "chunk.h"
+
+#include "configfile.h"
+#include "connections.h"
+
+#include "plugin.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -13,18 +24,8 @@
 
 #include <stdio.h>
 
-#include "response.h"
-#include "keyvalue.h"
-#include "log.h"
-#include "stat_cache.h"
-#include "chunk.h"
-
-#include "configfile.h"
-#include "connections.h"
-
-#include "plugin.h"
-
 #include "sys-socket.h"
+#include "version.h"
 
 int http_response_write_header(server *srv, connection *con) {
 	buffer *b;
@@ -43,6 +44,13 @@ int http_response_write_header(server *srv, connection *con) {
 	buffer_append_string_len(b, CONST_STR_LEN(" "));
 	buffer_append_string(b, get_http_status_name(con->http_status));
 
+	/* disable keep-alive if requested */
+	if (con->request_count > con->conf.max_keep_alive_requests || 0 == con->conf.max_keep_alive_idle) {
+		con->keep_alive = 0;
+	} else {
+		con->keep_alive_idle = con->conf.max_keep_alive_idle;
+	}
+
 	if (con->request.http_version != HTTP_VERSION_1_1 || con->keep_alive == 0) {
 		if (con->keep_alive) {
 			response_header_overwrite(srv, con, CONST_STR_LEN("Connection"), CONST_STR_LEN("keep-alive"));
@@ -64,7 +72,7 @@ int http_response_write_header(server *srv, connection *con) {
 
 		if (ds->value->used && ds->key->used &&
 		    0 != strncasecmp(ds->key->ptr, CONST_STR_LEN("X-LIGHTTPD-")) &&
-			0 != strcasecmp(ds->key->ptr, "X-Sendfile")) {
+			0 != strncasecmp(ds->key->ptr, CONST_STR_LEN("X-Sendfile"))) {
 			if (0 == strcasecmp(ds->key->ptr, "Date")) have_date = 1;
 			if (0 == strcasecmp(ds->key->ptr, "Server")) have_server = 1;
 			if (0 == strcasecmp(ds->key->ptr, "Content-Encoding") && 304 == con->http_status) continue;
@@ -104,7 +112,7 @@ int http_response_write_header(server *srv, connection *con) {
 
 	if (!have_server) {
 		if (buffer_is_empty(con->conf.server_tag)) {
-			buffer_append_string_len(b, CONST_STR_LEN("\r\nServer: " PACKAGE_NAME "/" PACKAGE_VERSION));
+			buffer_append_string_len(b, CONST_STR_LEN("\r\nServer: " PACKAGE_DESC));
 		} else if (con->conf.server_tag->used > 1) {
 			buffer_append_string_len(b, CONST_STR_LEN("\r\nServer: "));
 			buffer_append_string_encoded(b, CONST_BUF_LEN(con->conf.server_tag), ENCODING_HTTP_HEADER);
@@ -226,12 +234,6 @@ handler_t http_response_prepare(server *srv, connection *con) {
 			log_error_write(srv, __FILE__, __LINE__,  "sb", "URI-query    : ", con->uri.query);
 		}
 
-		/* disable keep-alive if requested */
-
-		if (con->request_count > con->conf.max_keep_alive_requests) {
-			con->keep_alive = 0;
-		}
-
 
 		/**
 		 *
